sampler fontTexture: register(s0);

uniform float currentTime;

float4 drawChar(float2 texCoord, int character, float2 coords, float2 size)
{
  float2 normalizedTexCoord = (texCoord - coords) / size;
  
  if(normalizedTexCoord.x >= 0.0f && normalizedTexCoord.y >= 0.0f && normalizedTexCoord.x < 1.0f && normalizedTexCoord.y < 1.0f)
  {
    character -= 32;
      
    float2 charCoord;

    charCoord.y = float(character / 32);
    charCoord.x = float(character) - charCoord.y * 32.0f;
    
    charCoord += normalizedTexCoord;
    charCoord *= float2(1.0f / 32.0f, 1.0f / 3.0f);

    return tex2D(fontTexture, charCoord);
  }
  
  return float4(0.0f, 0.0f, 0.0f, 1.0f);
}

float4 drawCircle(float2 texCoord, float2 center, float radius)
{
  if(length(texCoord - center) <= radius)
  {
    return float4(1.0f, 1.0f, 1.0f, 1.0f);
  }
  return float4(0.0f, 0.0f, 0.0f, 1.0f);
}

float4 drawLine(float2 texCoord, float2 start, float2 end, float width)
{
  float2 dir = end - start;
  start -= texCoord;
  float lambda = clamp(-1.0 * (start.x*dir.x + start.y*dir.y) / (dir.x * dir.x + dir.y * dir.y), 0.0, 1.0);
  
  if(length(start + lambda * dir) <= width)
  {
    return float4(1.0f, 1.0f, 1.0f, 1.0f);
  }
  return float4(0.0f, 0.0f, 0.0f, 1.0f);
}

float4 main(float2 texCoord: TEXCOORD0,
            float4 pos: POSITION
            ) : COLOR0
{
  float weight;
  float4 result = float4(0.0f, 0.0f, 0.1f, 1.0f);
  
  result += drawCircle(texCoord, float2(0.5f, 0.5f), 0.3f) * float4(1.0f, 1.0f, 0.0f, 1.0f);
  
  weight = length(drawLine(texCoord, float2(0.5f, 0.5f), float2(0.5f, 0.5f) + float2(cos(currentTime), sin(currentTime)) * 0.5f, 0.01f).xyz) * 0.5f;
  result = weight * float4(0.0f, 0.0f, 0.4f, 1.0f) + (1.0 - weight) * result;
  
  float4 text = float4(0.0f, 0.0f, 0.0f, 1.0f);
  
  float2 charSize = float2(0.05f, 0.05f);
  
  texCoord -= float2(0.2f, 0.2f);

  text += drawChar(texCoord, 'H', float2(0.00f, 0.0f), charSize);
  text += drawChar(texCoord, 'a', float2(0.05f, 0.0f), charSize);
  text += drawChar(texCoord, 'l', float2(0.10f, 0.0f), charSize);
  text += drawChar(texCoord, 'l', float2(0.15f, 0.0f), charSize);
  text += drawChar(texCoord, 'o', float2(0.20f, 0.0f), charSize);
  text += drawChar(texCoord, ',', float2(0.25f, 0.0f), charSize);
  
  text += drawChar(texCoord, 'P', float2(0.35f, 0.0f), charSize);
  text += drawChar(texCoord, 'B', float2(0.40f, 0.0f), charSize);
  text += drawChar(texCoord, '!', float2(0.45f, 0.0f), charSize);

  weight = length(text.xyz) * 0.5f;
  result = weight * float4(1.0f, 0.0f, 0.0f, 1.0f) + (1.0 - weight) * result;

  return result;
}